#
#  Name:    MpWx.py
#
#  Purpose: WxPython widgets for the MX control system.
#
#  Author:  William Lavender
#
#-----------------------------------------------------------------------
#
#  Copyright 2008-2009, 2011-2013, 2015 Illinois Institute of Technology
#
#  See the file "LICENSE" for information on usage and redistribution
#  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

import time
import wx
import Mp

#
#=============================================================================
#

class Value(wx.StaticText):

	def __init__( self, parent, server_record, field_name,
			function=None, args=None,
			id=-1, pos=wx.DefaultPosition,
			size=wx.DefaultSize,
			style=0, colour="blue", 
			name=wx.StaticTextNameStr, base=None ):

		wx.StaticText.__init__( self, parent, id=id,
			label="-----",
			pos=pos, size=size, style=style, name=name )

		self.parent = parent

		if ( ( base != 10 ) and ( base != 16 ) \
		    and ( base != 8 ) and ( base != None ) ):
			error_message = \
				"Only base 8, 10, and 16 are supported."

			raise ValueError, error_message

		self.base = base

		self.nf = Mp.Net( server_record, field_name )

		# Test for the existence of the network field.

		try:
			test = self.nf.get()

		except Mp.Not_Found_Error:
			self.SetLabel( "NOT FOUND" )
			self.Enable(False)
			return

		# Arrange for automatic updates of the value.

		if ( function == None ):
			function = _Value_update

		self.nf.add_to_value_changed_list( self, function, args )

		# FIXME: The following has no effect.  Why is that?

		self.SetForegroundColour( colour )

#----

def _Value_update( nf, widget, args, value ):

	if isinstance( value, list ):
		if ( len(value) == 1 ):
			value = value[0]

	if ( widget.base == 10 ):
		value = "%d" % value
	elif ( widget.base == 16 ):
		value = "%#x" % value
	elif ( widget.base == 8 ):
		value = "%#o" % value
	else:
		value = str( value )

	widget.SetLabel( value )

	widget.SetSize( widget.GetBestSize() )

	widget.Refresh()

#
#=============================================================================
#

class ValueEntry(wx.TextCtrl):

	def __init__( self, parent, server_record, field_name, \
			function=None, args=None, \
			id=-1, pos=wx.DefaultPosition, size=wx.DefaultSize, \
			style=0, name=wx.TextCtrlNameStr, \
			validator=wx.DefaultValidator ):

		# Adding wx.TE_PROCESS_ENTER to the style causes the
		# widget to generate wx.EVT_TEXT_ENTER events when
		# the Enter key is pressed.

		style = style | wx.TE_PROCESS_ENTER

		wx.TextCtrl.__init__( self, parent, id=id, \
			value=wx.EmptyString, pos=pos, size=size, \
			style=style, validator=validator )

		self.parent = parent

		self.nf = Mp.Net( server_record, field_name )

		# Test for the existence of the network field.

		try:
			test = self.nf.get()

		except Mp.Not_Found_Error:
			self.SetValue( "NOT FOUND" )
			self.Enable(False)
			return

		# Disable the widget if the network field is read only.

		read_only = self.nf.get_attribute( Mp.MXNA_READ_ONLY )

		if ( read_only ):
			self.Enable(False)

		# Arrange for automatic updates of the value.

		if ( function == None ):
			function = _ValueEntry_update

		self.nf.add_to_value_changed_list( self, function, args )

		# Typing keys in the entry field will change the color.

		self.Bind( wx.EVT_TEXT, self.OnText )

		# <Enter> keystrokes will change the value of the MX field.

		self.Bind( wx.EVT_TEXT_ENTER, self.OnEnter )

	def OnText( self, event ):
		self.SetBackgroundColour( "yellow" )

	def OnEnter( self, event ):

		value = self.GetValue().strip()

		self.nf.put( value )

		time.sleep(0.5)

		self.SetBackgroundColour( wx.NullColour )

#----

def _ValueEntry_update( nf, widget, args, value ):

	if isinstance( value, list ):
		if ( len(value) == 1 ):
			value = value[0]

	value = str( value )

	widget.SetValue( value )

	widget.SetBackgroundColour( wx.NullColour )

	#### widget.SetSize( widget.GetBestSize() )

	widget.Refresh()

#
#=============================================================================
#

class Choice(wx.Choice):

	def __init__( self, parent, server_record, field_name, \
			function=None, args=None, choices=None, \
			id=-1, pos=wx.DefaultPosition, size=wx.DefaultSize, \
			style=0, name=wx.ChoiceNameStr, \
			validator=wx.DefaultValidator ):

		wx.Choice.__init__( self, parent, id, \
			choices=choices, pos=pos, size=size, \
			style=style, validator=validator )

		self.parent = parent

		self.choices = choices

		self.nf = Mp.Net( server_record, field_name )

		# Test for the existence of the network field.

		try:
			test = self.nf.get()

		except Mp.Not_Found_Error:
			self.Enable(False)
			return

		# Disable the widget if the network field is read only.

		read_only = self.nf.get_attribute( Mp.MXNA_READ_ONLY )

		if ( read_only ):
			self.Enable(False)

		# Arrange for automatic updates of the value.

		if ( function == None ):
			function = _Choice_update

		self.nf.add_to_value_changed_list( self, function, args )

		# Selecting a choice will change the value of the MX field.

		self.Bind( wx.EVT_CHOICE, self.OnChoice )

	def OnChoice( self, event ):

		selection = self.GetSelection()

		selection_string = self.GetString( selection )

		value = float( selection_string )

		self.nf.put( value )

#----

def _Choice_update( nf, widget, args, value ):

	if isinstance( value, list ):
		if ( len(value) == 1 ):
			value = value[0]

	value = str( value )

	choice_list = widget.choices

	choice = -1

	for i in range( len(choice_list) ):
		if ( choice_list[i] == value ):
			choice = i
			break

	if ( choice >= 0 ):
		widget.SetSelection( choice )

#
#=============================================================================
#

class RadioBox(wx.RadioBox):

	def __init__( self, parent, server_record, field_name, \
			function=None, args=None, \
			id=-1, label=wx.EmptyString, \
			pos=wx.DefaultPosition, size=wx.DefaultSize, \
			choices=None, reverse_order=False, \
			majorDimension=0, style=wx.RA_HORIZONTAL, \
			validator=wx.DefaultValidator, \
			name=wx.RadioBoxNameStr ):

		wx.RadioBox.__init__( self, parent, id=id, label=label, \
			pos=pos, size=size, choices=choices, \
			majorDimension=majorDimension, style=style, \
			validator=validator, name=name )

		self.parent = parent

		self.choices = choices

		self.reverse_order = reverse_order

		self.nf = Mp.Net( server_record, field_name )

		# Test for the existence of the network field.

		try:
			test = self.nf.get()

		except Mp.Not_Found_Error:
			self.Enable(False)
			return

		# Disable the widget if the network field is read only.

		read_only = self.nf.get_attribute( Mp.MXNA_READ_ONLY )

		if ( read_only ):
			self.Enable(False)

		# Arrange for automatic updates of the value.

		if ( function == None ):
			function = _RadioBox_update

		self.nf.add_to_value_changed_list( self, function, args )

		# Selecting a choice will change the value of the MX field.

		self.Bind( wx.EVT_RADIOBOX, self.OnRadioBox )

	def OnRadioBox( self, event ):
		
		value = self.GetSelection()

		value = int( value )

		if ( self.reverse_order ):
			num_values = len(self.choices)
			value = num_values - 1 - value

		self.nf.put( value )

#----

def _RadioBox_update( nf, widget, args, value ):

	if isinstance( value, list ):
		if ( len(value) == 1 ):
			value = value[0]

	if ( widget.reverse_order ):
		num_values = len(widget.choices)
		value = num_values - 1 - value

	widget.SetSelection( value )

#
#=============================================================================
#

class Button(wx.Button):

	def __init__( self, parent,
			server_record, field_name,
			label=wx.EmptyString,
			value="",
			size=wx.DefaultSize ):

		wx.Button.__init__( self, parent,
			id=-1, label=label, size=size )

		self.nf = Mp.Net( server_record, field_name )

		# Test for the existence of the network field.

		try:
			test = self.nf.get()

		except Mp.Not_Found_Error:
			self.Enable(False)
			return

		self.value = value

		# Pressing the button sends the value to the network field.

		self.Bind( wx.EVT_BUTTON, self.OnButton )

	def OnButton( self, event ):

		self.nf.put( self.value )

